/**
   A simple generic implementation of the Stack abstract data type
   that uses a linked list implementation with an iterator.

   See pages 316 - 318 of the textbook.

   Here is what the Stack interface adds to the Container interface.
      public void push(T element);
      public T pop();
      public T peek();
*/
import java.util.Iterator;
import java.util.NoSuchElementException;

public class LinkedStack<T> extends Stack<T>
{
   /**
      This is an implementation of the Iterator interface
      that knows how to iterator over a LinkedStack.

      Since the LinkedStack class does not expose any information
      about its internal implementation, this Iterator must
      be a nested class within the LinkedStack class so that the
      iterator object has access to the internals of a LinkedStack
      object.
   */
   class LinkedStackIterator implements Iterator<T>
   {
      Node<T> cursor;

      public LinkedStackIterator( )
      {
         // Here is where we make use of the fact that
         // this iterator is nested inside of the LinkedStack
         // class. We have access to the private field
         // first.
         this.cursor = first;
      }

      public boolean hasNext()
      {
         return (this.cursor != null);
      }

      public T next()
      {
         if ( hasNext() )
         {
            T temp = this.cursor.data;
            this.cursor = this.cursor.next;
            return temp;
         }
         else
            throw new NoSuchElementException();
      }

      public void remove()
      {
         throw new UnsupportedOperationException();
      }
   }//LinkedStackIterator


   // A private field in the LinkedStack class.
   private Node<T> first;

   public LinkedStack()
   {
      super();
      first = null;
   }


   public void add(T element)
   {
      push(element);
   }


   public void push(T element)
   {
      Node<T> node = new Node<T>(element, first);
      first = node;
      size++;
   }


   public T pop()
   {
      T data = first.data;
      first = first.next;
      size--;
      return data;
   }


   public T peek()
   {
      return first.data;
   }


   // Provide an implementation of the Iterable interface.
   public Iterator<T> iterator()
   {
      return new LinkedStackIterator();
   }

}//LinkedStack
